import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import { Text } from "./Text";
import { buttonReset } from "./Button";
// import {VisuallyHidden} from "@reach/visually-hidden";
import * as VisuallyHidden from '@radix-ui/react-visually-hidden';
import PropTypes from "prop-types";
import { alpha } from "./Theme/colors";
import { useSpring, animated } from "@react-spring/web";
import { usePrevious } from "./Hooks/previous";
import { Badge } from "./Badge";
import { useTheme } from "./Theme/Providers";
import { IconWrapper } from "./IconWrapper";
// import computeScrollIntoView from "compute-scroll-into-view";
import { noOp } from "./misc/noop";
import { useMeasure } from "./Hooks/use-measure";
import { mergeRefs } from "./Hooks/merge-refs";
import { TouchRefComp } from "./Touchable";
import cx from "classnames";
import scrollIntoView from 'scroll-into-view-if-needed';
const hideScrollbar = css `
  ::-webkit-scrollbar {
    height: 0;
    width: 0;
  }
`;
/**
 * Provide a tabbed interface 多个子页面按钮切换器容器.
 */
export const Tabs = ({ children, variant = "default", slider: enableSlider = true, value, animationConfig = { tension: 190, friction: 15, mass: 0.2 }, onChange, ...other }) => {
    const theme = useTheme();
    const boundary = React.useRef(null);
    const tablist = React.useRef(null);
    const refs = React.useRef(new Map());
    const dark = theme.colors.mode === "dark";
    // We store the 'value' prop to determine when we should actually
    // animated our slider. ie., only when the index changes.
    const [slider, setSlider] = React.useState({
        right: 0,
        left: 0,
        value
    });
    const [showSlider, setShowSlider] = React.useState(false);
    const previousSlider = usePrevious(slider);
    // const scrollRef = React.useRef<ScrollViewHandles>(null);
    const { bounds } = useMeasure(boundary);
    //boundary底下的滚动才能显示出来。 target：目标一个按钮。
    React.useEffect(() => {
        const target = refs.current.get(value);
        if (target) {
            scrollIntoView(target, {
                behavior: 'smooth',
                block: 'end',
                boundary: boundary.current,
            });
            //scrollRef.current!.scrollTo(left);
        }
    }, [value]);
    // measure our elements
    React.useEffect(() => {
        const target = refs.current.get(value);
        const container = tablist.current;
        if (target) {
            const cRect = container.getBoundingClientRect();
            // when container is `display: none`, width === 0.
            // ignore this case
            if (cRect.width === 0) {
                return;
            }
            const tRect = target.getBoundingClientRect();
            const left = tRect.left - cRect.left;
            const right = cRect.right - tRect.right;
            setSlider({
                value,
                left: left + 8,
                right: right + 8
            });
            setShowSlider(true);
        }
    }, [value, bounds]);
    const spring = useSpring({
        left: slider.left + "px",
        right: slider.right + "px",
        immediate: previousSlider ? previousSlider.value === slider.value : false,
        config: animationConfig // default friction is 160: speed up our animation slightly
    });
    // Thanks to Ryan Florence for this code
    // https://github.com/reach/reach-ui/blob/master/packages/tabs/src/index.js
    function onKeyDown(e) {
        const enabledIndexes = React.Children.map(children, (child, index) => {
            return child.props.disabled === true ? null : index;
        }).filter(index => index != null);
        const enabledSelectedIndex = enabledIndexes.indexOf(value);
        switch (e.key) {
            case "ArrowRight": {
                const nextEnabledIndex = (enabledSelectedIndex + 1) % enabledIndexes.length;
                const nextIndex = enabledIndexes[nextEnabledIndex];
                if (typeof nextIndex === "number") {
                    onChange(nextIndex);
                }
                break;
            }
            case "ArrowLeft": {
                const count = enabledIndexes.length;
                const nextEnabledIndex = (enabledSelectedIndex - 1 + count) % count;
                const nextIndex = enabledIndexes[nextEnabledIndex];
                if (typeof nextIndex === "number") {
                    onChange(nextIndex);
                }
                break;
            }
            case "Home": {
                onChange(0);
                break;
            }
            case "End": {
                onChange(React.Children.count(children) - 1);
                break;
            }
        }
    }
    return (_jsx("div", { className: "Tabs", ref: boundary, css: {
            overflowY: "hidden",
            boxShadow: dark ? `0px 3px 2px rgba(0,0,0,0.15)` : "none"
        }, ...other, children: _jsx("div", { className: "Tabs__container", css: [
                {
                    width: "100%",
                    whiteSpace: "nowrap",
                    paddingBottom: "20px",
                    marginBottom: "-20px",
                    //        overflowX
                },
                hideScrollbar
            ], children: _jsxs("div", { className: "Tabs__tablist", role: "tablist", ref: tablist, onKeyDown: onKeyDown, css: {
                    display: variant === "evenly-spaced" ? "flex" : "inline-block",
                    position: "relative",
                    verticalAlign: "bottom",
                    "& button": {
                        flex: variant === "evenly-spaced" ? "1" : undefined
                    }
                }, children: [React.Children.map(children, (child, i) => {
                        return React.cloneElement(child, {
                            isActive: i === value,
                            ref: (el) => {
                                refs.current.set(i, el);
                            },
                            onParentSelect: () => {
                                onChange(i);
                            }
                        });
                    }), enableSlider && showSlider && (_jsx(animated.div, { className: "Tabs__slider", style: spring, css: {
                            height: "4px",
                            borderTopRightRadius: "8px",
                            borderTopLeftRadius: "8px",
                            marginLeft: theme.spaces.sm,
                            marginRight: theme.spaces.sm,
                            bottom: 0,
                            position: "absolute",
                            background: dark ? "white" : theme.colors.text.selected
                        } }))] }) }) }));
};
Tabs.propTypes = {
    value: PropTypes.number.isRequired,
    onChange: PropTypes.func,
    slider: PropTypes.bool,
    variant: PropTypes.oneOf(["default", "evenly-spaced"]),
    children: PropTypes.arrayOf(PropTypes.element),
    animationConfig: PropTypes.shape({
        tension: PropTypes.number,
        friction: PropTypes.number,
        mass: PropTypes.number
    })
};
/**
 * A clickable tab item
 * 上级Tabs 将会隐形传递一个ref参数给 Tab的，所以只能上React.forwardRef()
 */
export const Tab = React.forwardRef(({ onParentSelect, isActive, id, component: Component = "button", badge, onPress = noOp, children, ...other }, forwarded) => {
    const theme = useTheme();
    const dark = theme.colors.mode === "dark";
    /**
     * Determine text color
     */
    function getTextColor(isDark) {
        if (isDark) {
            return isActive ? "rgba(255,255,255,1)" : "rgba(255,255,255,0.65)";
        }
        return isActive ? theme.colors.text.selected : theme.colors.text.muted;
    }
    /**
     * Determine badge colors
     */
    function getBadgeColors(isDark) {
        let background = "white";
        let color = "white";
        if (isDark) {
            background = isActive ? "white" : "rgba(255,255,255,0.65)";
            color = isActive
                ? theme.colors.text.selected
                : theme.colors.text.selected;
        }
        else {
            background = isActive
                ? theme.colors.text.selected
                : theme.colors.scales.gray[6];
            color = isActive ? "white" : theme.colors.text.selected;
        }
        return {
            background,
            color
        };
    }
    const mounted = React.useRef(false);
    //代表ref是按钮区域;  forwarded是开放给上一级的{forwardRef.Ref}
    //Tabs会传递给Tab的forwarded是(el: HTMLButtonElement某元素| null) => {refs.current!.set(i, el);点选定这个tab} 传递ref()实际运行该函数。
    const ref = React.useRef(null);
    /**
     * Handle focus side-effects
     */
    React.useEffect(() => {
        // don't autofocus if mounting
        if (!mounted.current) {
            mounted.current = true;
            return;
        }
        if (!isActive) {
            return;
        }
        if (ref.current) {
            ref.current.focus();
        }
    }, [isActive]);
    const onPressFn = React.useCallback(e => {
        onPress(e);
        if (onParentSelect) {
            onParentSelect();
        }
    }, [onParentSelect, onPress]);
    return (_jsxs(TouchRefComp, { className: cx("Tab", {
            "Tab--active": isActive
        }), component: Component, terminateOnScroll: false, onPress: onPressFn, delay: 0, css: [
            buttonReset,
            {
                padding: `10px ${theme.spaces.md}`,
                [theme.mediaQueries.lg]: {
                    padding: `10px ${theme.spaces.lg}`
                },
                cursor: "pointer",
                color: getTextColor(dark),
                transition: "background 0.35s ease",
                background: "transparent",
                "& span": {
                    transition: "color 0.25s cubic-bezier(0.35,0,0.25,1)"
                },
                "& svg": {
                    transition: "stroke 0.35s cubic-bezier(0.35,0,0.25,1)",
                    stroke: getTextColor(dark) + " !important"
                },
                ":focus": {
                    outline: "none",
                    background: dark
                        ? alpha(theme.colors.background.tint2, 0.15)
                        : theme.colors.background.tint2
                },
                ":focus:not([data-focus-visible-added])": {
                    outline: "none",
                    background: "transparent"
                },
                "&.Touchable--active": {
                    color: dark
                        ? "rgba(255,255,255,0.4)"
                        : alpha(theme.colors.text.selected, 0.4),
                    "& svg": {
                        stroke: (dark
                            ? "rgba(255,255,255,0.3)"
                            : alpha(theme.colors.text.selected, 0.4)) + " !important"
                    }
                }
            }
        ], ref: mergeRefs(ref, forwarded), role: "tab", id: id + "-tab", "aria-controls": id, "aria-selected": isActive, tabIndex: isActive ? 0 : -1, ...other, children: [typeof children === "string" ? (_jsx(Text, { wrap: false, component: "span", variant: "subtitle", css: {
                    fontSize: theme.fontSizes[0],
                    fontWeight: 600,
                    color: "inherit",
                    transition: "color 0.25s cubic-bezier(0.35,0,0.25,1)"
                }, children: children })) : (children), badge && (_jsx("div", { css: { display: "inline", marginLeft: theme.spaces.sm }, children: typeof badge === "string" || typeof badge === "number" ? (_jsx(Badge, { css: getBadgeColors(dark), children: badge })) : (badge) }))] }));
});
Tab.displayName = "Tab";
Tab.propTypes = {
    id: PropTypes.string.isRequired,
    children: PropTypes.node,
    badge: PropTypes.node
};
export const TabPanel = ({ id, ...other }) => (_jsx("div", { id: id, role: "tabpanel", tabIndex: 0, "aria-labelledby": id + "-tab", ...other }));
TabPanel.propTypes = {
    id: PropTypes.string.isRequired
};
export const TabIcon = ({ icon, label, size = "lg" }) => {
    return (_jsxs("div", { css: { display: "flex", justifyContent: "center" }, children: [_jsx(IconWrapper, { size: size, children: icon }), _jsx(VisuallyHidden.Root, { children: label })] }));
};
TabIcon.propTypes = {
    icon: PropTypes.node.isRequired,
    label: PropTypes.string.isRequired,
    size: PropTypes.oneOf(["xs", "sm", "md", "lg", "xl"])
};
//# sourceMappingURL=Tabs.js.map