import { jsx as _jsx } from "react/jsx-runtime";
import * as React from "react";
import defaultTheme from ".";
/**
 * API:
 *
 * You can create custom themes using Object.assign:
 *
 *
 * Pass that theme to the theme provider. A theme should always
 * provide a dark and light color mode.
 *
 * <ThemeProvider theme={customTheme}><App /></ThemeProvider>
 *
 *
 * When you want to use a dark mode:
 *
 * function App () {
 *  return <Dark><SomeContent /></Dark>
 * }
 *
 *
 * To consume a theme, use `useTheme` hook
 *
 * const theme = useTheme()
 * // theme.colors.text.default (will be dark or light depending on the mode)
 * // theme.colors.mode === 'dark' or 'light'
 */
const ThemeContext = React.createContext(defaultTheme);
export const ThemeProvider = ({ theme = defaultTheme, children }) => {
    return (_jsx(ThemeContext.Provider, { value: theme, children: children }));
};
/**
 * A hook for consuming the theme
 */
export function useTheme() {
    return React.useContext(ThemeContext);
}
/**
 * 去掉forwardRef，改成普通函数组件。
 */
const ColorMode = ({ colors, children, ...other }) => {
    const theme = useTheme();
    //依照colors参数调整Theme结果； memo is necessary to prevent unnecessary rerenders
    // https://reactjs.org/docs/context.html#caveats
    const adjustedTheme = React.useMemo(() => mergeColors(theme, colors), [
        theme,
        colors
    ]);
    //其实这里向.Children.only(children)传递的 ref 实际没有用处；
    //本组件ThemeContext才是核心，把组件包裹下的子孙组件提供一致的 colors Theme;
    //这个版本调用 ? children(adjustedTheme) 就会报错？ 仅仅是编译器报错?
    //只能在forwardRef版本中才能使用? children(adjustedTheme)这样的 函数式儿子。
    //本组件底下包裹的子组件是 函数式儿子举例说明  {(theme: Theme) => (  <div  theme.colors >  )}  这样子的，非正常儿孙组件。
    return (_jsx(ThemeContext.Provider, { value: adjustedTheme, children: typeof children === "function"
            ? children(adjustedTheme)
            : React.cloneElement(React.Children.only(children), {
                ...other
            }) }));
};
/**
 * 旧版传递 ref , 这里Ref只是接续传递，非自身必须要的；
 */
const ColorRefMode = React.forwardRef(({ colors, children, ...other }, ref) => {
    const theme = useTheme();
    // memo is necessary to prevent unnecessary rerenders
    // https://reactjs.org/docs/context.html#caveats
    const adjustedTheme = React.useMemo(() => mergeColors(theme, colors), [
        theme,
        colors
    ]);
    //这个forwardRef版本才能调用 ? children(adjustedTheme)   函数式儿子；
    //本组件底下包裹的子组件是 函数式儿子举例说明  {(theme: Theme) => (  <div  theme.colors >  )}  这样子的，非正常儿孙组件。
    //非Ref版本,只能添加类型转换 ? (children as RenderCallbackType)(adjustedTheme) ;仅仅是编译器报错?
    return (_jsx(ThemeContext.Provider, { value: adjustedTheme, children: typeof children === "function"
            ? children(adjustedTheme)
            : React.cloneElement(React.Children.only(children), {
                ref,
                ...other
            }) }));
});
ColorMode.displayName = "ColorMode";
function mergeColors(theme, colors) {
    return {
        ...theme,
        shadows: colors.shadows,
        colors
    };
}
export const LightMode = ({ children, ...other }) => {
    const theme = useTheme();
    return (_jsx(ColorMode, { colors: theme.modes.light, ...other, children: children }));
};
/**
 * 旧版传递 ref, 这里Ref只是接续传递，非自身必须要的；
 */
export const LightRefMode = React.forwardRef(({ children, ...other }, ref) => {
    const theme = useTheme();
    return (_jsx(ColorRefMode, { colors: theme.modes.light, ref: ref, ...other, children: children }));
});
LightMode.displayName = "LightMode";
/**
 * Provide a dark theme
 */
export const DarkMode = ({ children, ...other }) => {
    const theme = useTheme();
    return (_jsx(ColorMode, { colors: theme.modes.dark, ...other, children: children }));
};
/**
 * 旧版传递 ref, 这里Ref只是接续传递，非自身必须要的；
 * DarkRefMode Light#Mode 需要交替嵌套； 可以传递下去给子孙组件的useTheme() ThemeContext.Provider。
 * 只能两组： 2选择1的颜色？ 白天 pk 黑夜。
 */
export const DarkRefMode = React.forwardRef(({ children, ...other }, ref) => {
    const theme = useTheme();
    return (_jsx(ColorRefMode, { colors: theme.modes.dark, ref: ref, ...other, children: children }));
});
DarkMode.displayName = "DarkMode";
//# sourceMappingURL=Providers.js.map